﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using OpenTK.Graphics.OpenGL;
using OpenTK;
using System.Drawing;

namespace LengthwiseShootingGame
{
	public class Boss : EnemyBase
	{
		public Boss(float x, float y, float radius, int life, int score)
		{
			X = x;
			Y = y;
			Z = InitZ;

			IsDead = false;
			Score = score;
			MaxLife = life;
			Life = life;
			Radius = radius;

			VecX = 0.1f;
			VecY = Form1.Speed;

			if(Textures.Count == 0)
				Textures = CreateTextures();
		}
		float InitZ = 10f;
		float Z = 0;

		public int MaxLife = 0;

		bool isRight = true;
		public bool IsReady = false;

		static List<int> Textures = new List<int>();

		protected override List<Bitmap> GetBitmaps()
		{
			Bitmap bitmap = Properties.Resources.sprite2;

			List<Bitmap> bitmaps = new List<Bitmap>();

			// ボス敵
			// 0
			{
				Bitmap bitmap1 = new Bitmap(140, 156);
				Graphics g = Graphics.FromImage(bitmap1);
				g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
				g.DrawImage(bitmap, new Rectangle(0, 0, 140, 156), new Rectangle(337, 300, 140, 156), GraphicsUnit.Pixel);
				g.Dispose();
				bitmaps.Add(bitmap1);
			}
			return bitmaps;
		}

		public override void Move()
		{
			if(Z > 2)
			{
				Z -= 0.1f;
				Y += Form1.Speed;
				return;
			}

			IsReady = true;
			Z = 2f;

			if(X > 3)
				isRight = false;
			if(X < -3)
				isRight = true;

			if(isRight)
				VecX = 0.1f;
			else
				VecX = -0.1f;

			// 周囲に弾幕を張るように弾丸を配置
			Shot();

			// Lifeが半分以下になったらザコ敵をつくって参戦させる。
			if(Life > 0 && Life < MaxLife / 4 && MoveCount % 16 == 0)
			{
				int r = Random.Next(800);
				float x = 1f * r / 100 - 4f;
				Enemies.Add(new Enemy2(x, Y + 10, 0.5f, 1, 10));
			}
			else if(Life > 0 && Life < MaxLife / 2 && MoveCount % 64 == 0)
			{
				int r = Random.Next(800);
				float x = 1f * r / 100 - 4f;
				Enemies.Add(new Enemy2(x, Y + 10, 0.5f, 1, 10));
			}

			base.Move();
		}

		List<BossEnemyBurret> bossEnemyBurrets = new List<BossEnemyBurret>();

		void Shot()
		{
			int i = MoveCount % 31;
			if(i != 30)
			{
				int angle = i * 12 + 180;

				double rad = angle * Math.PI / 180;
				float vecX = (float)Math.Cos(rad) * 0.2f;
				float vecY = (float)Math.Sin(rad) * 0.2f + Form1.Speed;

				float x = X + (float)Math.Cos(rad) * 3;
				float y = Y + (float)Math.Sin(rad) * 3;

				BossEnemyBurret burret = new BossEnemyBurret(this, x, y, vecX, vecY);
				Form1.EnemyBurrets.Add(burret);
				bossEnemyBurrets.Add(burret);
			}
			if(i == 30)
			{
				foreach(BossEnemyBurret burret in bossEnemyBurrets)
					burret.IsMove = true;

				bossEnemyBurrets.Clear();
			}
		}

		public List<EnemyBase> Enemies = new List<EnemyBase>();

		public override void Draw()
		{
			GL.PushMatrix();
			{
				GL.Translate(X, Y, Z);

				GL.Material(MaterialFace.Front, MaterialParameter.Ambient, Color.White);
				GL.Material(MaterialFace.Front, MaterialParameter.Diffuse, Color.White);

				GL.BindTexture(TextureTarget.Texture2D, Textures[0]);

				GL.Begin(BeginMode.Quads);
				{
					GL.Normal3(Vector3.UnitY);
					GL.TexCoord2(1, 0);
					GL.Vertex3(Radius, 0, Radius);
					GL.TexCoord2(0, 0);
					GL.Vertex3(-Radius, 0, Radius);
					GL.TexCoord2(0, 1);
					GL.Vertex3(-Radius, 0, -Radius);
					GL.TexCoord2(1, 1);
					GL.Vertex3(Radius, 0, -Radius);
				}
				GL.End();
				GL.BindTexture(TextureTarget.Texture2D, 0);
			}
			GL.PopMatrix();
		}

	}

	public class BossEnemyBurret : EnemyBurret
	{
		public BossEnemyBurret(Boss boss, float x, float y, float vecX, float vecY) : base(x, y, vecX, vecY)
		{
			_boss = boss;
			oldBossX = boss.X;
		}

		public bool IsMove = false;
		Boss _boss = null;
		float oldBossX = 0;

		public override void Move()
        {
			if(IsMove)
			{
				X += VecX;
				Y += VecY;
			}
			else
			{
				// 前回のボスのX座標との差分
				float difBossX = _boss.X - oldBossX;

				Y += Form1.Speed;
				X = X + difBossX;

				oldBossX = _boss.X;
			}
			MoveCount++;
		}

	}
}
