﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using OpenTK.Graphics.OpenGL;
using OpenTK;
using System.Drawing;

namespace LengthwiseShootingGame
{
	public class Character
	{
		public float X = 0;
		public float Y = 0;

		public float VecX = 0;
		public float VecY = 0;

		public bool IsDead = false;

		public float Radius
		{
			get;
			protected set;
		}

		int _life = 0;

		public int Life
		{
			get { return _life; }
			set
			{
				_life = value;
				if(_life <= 0)
					IsDead = true;
			}
		}

		protected int MoveCount = 0;

		public virtual void Move()
		{
			X += VecX;
			Y += VecY;

			MoveCount++;
		}

		public virtual void Draw()
		{
		}

		protected virtual List<Bitmap> GetBitmaps()
		{
			List<Bitmap> bitmaps = new List<Bitmap>();
			return bitmaps;
		}

		protected List<int> CreateTextures()
		{
			Console.WriteLine("CreateTextures()");
			List<int> textures = new List<int>();

			// テクスチャ有効化
			GL.Enable(EnableCap.Texture2D);

			GL.AlphaFunc(AlphaFunction.Gequal, 0.5f);
			GL.Enable(EnableCap.AlphaTest);

			List<Bitmap> bitmaps = GetBitmaps();

			foreach(Bitmap bitmap0 in bitmaps)
			{
				// 領域確保
				int texture;
				GL.GenTextures(1, out texture);
				textures.Add(texture);

				// アクティブ設定
				GL.BindTexture(TextureTarget.Texture2D, texture);

				System.Drawing.Imaging.BitmapData data = bitmap0.LockBits(
					   new Rectangle(0, 0, bitmap0.Width, bitmap0.Height), // 画像全体をロック
					   System.Drawing.Imaging.ImageLockMode.ReadOnly, // 読み取り専用
					   System.Drawing.Imaging.PixelFormat.Format32bppArgb);

				// テクスチャ領域にピクセルデータを貼り付ける
				GL.TexImage2D(
					TextureTarget.Texture2D,
					0,
					PixelInternalFormat.Rgba,
					data.Width,
					data.Height,
					0,
					PixelFormat.Bgra, // Rgbaにすると赤と青が反転するので注意
					PixelType.UnsignedByte,
					data.Scan0);

				// ロックをしたら必ずアンロックする
				bitmap0.UnlockBits(data);

				GL.GenerateMipmap(GenerateMipmapTarget.Texture2D);

				GL.BindTexture(TextureTarget.Texture2D, 0);
			}
			return textures;
		}
	}


	public class EnemyBase: Character
	{
		public EnemyBase()
		{
			
		}

		int _life = 0;

		new public int Life
		{
			get { return _life; }
			set
			{
				_life = value;
				if(_life <= 0)
				{
					IsDead = true;
					Form1.Score += Score;
				}
			}
		}

		static protected Random Random = new Random(); 

		public int Score = 0;
	}
}
