﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using OpenTK.Graphics.OpenGL;
using OpenTK;
using System.Drawing;

namespace LengthwiseShootingGame
{
	public class Enemy1 : EnemyBase
	{
		public Enemy1(float x, float y, float radius, int life, int score)
		{
			X = x;
			Y = y;
			Radius = radius;
			IsDead = false;
			Score = score;
			Life = life;

			VecY = -0.3f;

			if(Textures.Count == 0)
				Textures = CreateTextures();
		}

		static List<int> Textures = new List<int>();
		float BurretSpeed = 0.2f;

		protected override List<Bitmap> GetBitmaps()
		{
			Bitmap bitmap = Properties.Resources.sprite2;

			List<Bitmap> bitmaps = new List<Bitmap>();

			// ザコ敵（ピンク色）0
			{
				Bitmap bitmap1 = new Bitmap(25, 28);
				Graphics g = Graphics.FromImage(bitmap1);
				g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.NearestNeighbor;
				g.DrawImage(bitmap, new Rectangle(0, 0, 25, 28), new Rectangle(4, 94, 25, 28), GraphicsUnit.Pixel);
				g.Dispose();
				bitmaps.Add(bitmap1);
			}
			// ザコ敵（ピンク色）1
			{
				Bitmap bitmap1 = new Bitmap(25, 28);
				Graphics g = Graphics.FromImage(bitmap1);
				g.DrawImage(bitmap, new Rectangle(0, 0, 25, 28), new Rectangle(36, 94, 25, 28), GraphicsUnit.Pixel);
				g.Dispose();
				bitmaps.Add(bitmap1);
			}
			// ザコ敵（ピンク色）2
			{
				Bitmap bitmap1 = new Bitmap(25, 28);
				Graphics g = Graphics.FromImage(bitmap1);
				g.DrawImage(bitmap, new Rectangle(0, 0, 25, 28), new Rectangle(68, 94, 25, 28), GraphicsUnit.Pixel);
				g.Dispose();
				bitmaps.Add(bitmap1);
			}
			// ザコ敵（ピンク色）3
			{
				Bitmap bitmap1 = new Bitmap(25, 28);
				Graphics g = Graphics.FromImage(bitmap1);
				g.DrawImage(bitmap, new Rectangle(0, 0, 25, 28), new Rectangle(99, 94, 25, 28), GraphicsUnit.Pixel);
				g.Dispose();
				bitmaps.Add(bitmap1);
			}
			return bitmaps;
		}


		bool doesAway = false;

		public override void Move()
		{
			Jiki jiki = Form1.Jiki;
			if(jiki == null)
				return;

			if(Y < Form1.Jiki.Y + 5)
			{
				doesAway = true;
				if(jiki.X < X)
					VecX = 0.1f;
				else
					VecX = -0.1f;
				VecY = 2.0f;

				float x = Form1.Jiki.X - X;
				float y = Form1.Jiki.Y - Y;

				if(x == 0)
					x = 0.001f;

				double angle1 = Math.Atan2(y , x);
				double angle2 = angle1 + 0.3f;
				double angle3 = angle1 - 0.3f;
				Form1.EnemyBurrets.Add(new EnemyBurret(X, Y, (float)Math.Cos(angle1) * BurretSpeed, (float)Math.Sin(angle1) * BurretSpeed + Form1.Speed));
				Form1.EnemyBurrets.Add(new EnemyBurret(X, Y, (float)Math.Cos(angle2) * BurretSpeed, (float)Math.Sin(angle3) * BurretSpeed + Form1.Speed));
				Form1.EnemyBurrets.Add(new EnemyBurret(X, Y, (float)Math.Cos(angle3) * BurretSpeed, (float)Math.Sin(angle3) * BurretSpeed + Form1.Speed));
			}
			
			if(!doesAway && MoveCount % 8 == 0 && Random.Next(2) == 0)
			{
				if(jiki.X < X)
					VecX = -0.1f;
				else
					VecX = 0.1f;
			}

			base.Move();
		}

		public override void Draw()
		{
			GL.PushMatrix();
			{
				GL.Translate(X, Y, 0);

				GL.Material(MaterialFace.Front, MaterialParameter.Ambient, Color.White);
				GL.Material(MaterialFace.Front, MaterialParameter.Diffuse, Color.White);

				int i = MoveCount % 16;
				if(i < 4)
					GL.BindTexture(TextureTarget.Texture2D, Textures[0]);
				else if(i < 8)
					GL.BindTexture(TextureTarget.Texture2D, Textures[1]);
				else if(i < 12)
					GL.BindTexture(TextureTarget.Texture2D, Textures[2]);
				else if(i < 16)
					GL.BindTexture(TextureTarget.Texture2D, Textures[3]);

				GL.Begin(BeginMode.Quads);
				{
					GL.Normal3(Vector3.UnitY);
					GL.TexCoord2(1, 0);
					GL.Vertex3(Radius, 0, Radius);
					GL.TexCoord2(0, 0);
					GL.Vertex3(-Radius, 0, Radius);
					GL.TexCoord2(0, 1);
					GL.Vertex3(-Radius, 0, -Radius);
					GL.TexCoord2(1, 1);
					GL.Vertex3(Radius, 0, -Radius);
				}
				GL.End();
				GL.BindTexture(TextureTarget.Texture2D, 0);
			}
			GL.PopMatrix();
		}
	}
}
