﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using OpenTK;
using OpenTK.Graphics.OpenGL;

namespace LengthwiseShootingGame
{
	public partial class Form1 : Form
	{
		Timer Timer = new Timer();
		Random Random = new Random();

		static public Jiki Jiki = null;
		static public float BaseY = 0;

		float InitEyeY = -6f;
		float InitEyeZ = 5f;
		float InitTargetY = 7f;
		static public float Speed = 0.2f;

		bool IsBattleAntiBoss = false;

		static public int Score = 0;
		List<JikiBurret> JikiBurrets = new List<JikiBurret>();
		List<EnemyBase> Enemies = new List<EnemyBase>();
		static public List<EnemyBurret> EnemyBurrets = new List<EnemyBurret>();

		int oldTextureForJikiLife = 0;
		int oldTextureForScore = 0;

		List<Explosion> Explosions = new List<Explosion>();

		public Form1()
		{
			InitializeComponent();

			Timer.Interval = 30;
			Timer.Tick += Timer_Tick;
			Timer.Start();
		}

		private void Timer_Tick(object sender, EventArgs e)
		{
			Update();

			if(Jiki != null && !Jiki.IsDead)
			{
				if(!IsBattleAntiBoss && bgm1EndTime < DateTime.Now)
					PlayBGM1();
				if(IsBattleAntiBoss && bgm2EndTime < DateTime.Now)
					PlayBGM2();
			}
			glControl.Refresh();
		}

		/// <summary>
		/// コントロールがロードされたら初期化をする OK
		/// </summary>
		private void glControlEx1_Load(object sender, EventArgs e)
		{
			GL.ClearColor(glControl.BackColor);

			// Projection の設定
			SetProjection();

			// デプスバッファの使用
			GL.Enable(EnableCap.DepthTest);

			glControl.Refresh();
		}

		/// <summary>
		/// Projection の設定
		/// </summary>
		private void SetProjection()
		{
			// ビューポートの設定
			GL.Viewport(0, 0, glControl.Width, glControl.Height);

			// 視体積の設定
			GL.MatrixMode(MatrixMode.Projection);
			Matrix4 proj = Matrix4.CreatePerspectiveFieldOfView(MathHelper.PiOver4, (float)glControl.Size.Width / (float)glControl.Size.Height, 0.01f, 40.0f);
			GL.LoadMatrix(ref proj);

			// MatrixMode を元に戻す
			GL.MatrixMode(MatrixMode.Modelview);
		}

		private void glControl_Resize(object sender, EventArgs e)
		{
			SetProjection();

			// 再描画
			glControl.Refresh();
		}

		/// <summary>
		/// OL
		/// </summary>
		void SetSight()
		{
			// 視界の設定
			Vector3 eye = new Vector3(0, InitEyeY + BaseY, InitEyeZ);
			Vector3 target = new Vector3(0, InitTargetY + BaseY, 0);
			Vector3 up = Vector3.UnitY;

			Matrix4 look = Matrix4.LookAt(eye, target, up);
			GL.LoadMatrix(ref look);
		}

		new void Update()
		{
			BaseY += Speed;

			if(Jiki != null && !Jiki.IsDead)
			{
				Jiki.Move();

				if(BaseY < 300)
				{
					CreateEnemies();
				}
				else
				{
					if(Enemies.Count == 0 && !IsBattleAntiBoss)
					{
						// boss登場
						BeginBattleAntiBoss();
					}
				}
			}

			MoveEnemies();

			MoveJikiBurrets();
			MoveEnemyBurrets();

			HitJudeJikiBurrets();
			HitJudeEnemyBurrets();
			CrashJudeEnemies();

			MoveExplosions();

			JikiBurrets = JikiBurrets.Where(x => !IsOutOfField(x) && !x.IsDead).ToList();
			Enemies = Enemies.Where(x => !IsOutOfField(x) && !x.IsDead).ToList();
			EnemyBurrets = EnemyBurrets.Where(x => !IsOutOfField(x) && !x.IsDead).ToList();
			Explosions = Explosions.Where(x => !x.IsDead).ToList();
		}

		void BeginBattleAntiBoss()
		{
			IsBattleAntiBoss = true;
			Enemies.Add(new Boss(0, 10 + BaseY, 3, 500, 1000));
			PlayBGM2();
		}

		void MoveExplosions()
		{
			foreach(Explosion explosion in Explosions)
			{
				explosion.Move();
			}
		}

		void HitJudeEnemyBurrets()
		{
			foreach(EnemyBurret burret in EnemyBurrets)
			{
				if(Jiki == null || Jiki.IsDead || Jiki.IsMuteki)
					return;

				if(Math.Pow(burret.Radius + Jiki.Radius, 2) > Math.Pow(burret.X - Jiki.X, 2) + Math.Pow(burret.Y - Jiki.Y, 2))
				{
					OnJikiDamage(burret);
					return;
				}
			}
		}

		void CrashJudeEnemies()
		{
			foreach(EnemyBase enemy in Enemies)
			{
				if(Jiki == null || Jiki.IsDead || Jiki.IsMuteki)
					return;

				if(Math.Pow(enemy.Radius + Jiki.Radius, 2) > Math.Pow(enemy.X - Jiki.X, 2) + Math.Pow(enemy.Y - Jiki.Y, 2))
				{
					OnJikiDamage(enemy);
					return;
				}
			}
		}

		void OnJikiDamage(Character enemyChar)
		{
			// Lifeが0になって自機が死亡した場合は大爆発
			// そうでない場合は立て続けにやられないためにしばらく無敵状態にする。

			Jiki.Life--;
			enemyChar.IsDead = true;
			Explosions.Add(new Explosion(Jiki.X, Jiki.Y, 0, 0, 0.5f));

			PlaySeDamage();
			Jiki.IsBackColorRed = true;

			if(Jiki.IsDead)
			{
				BigExplosion(Jiki.X, Jiki.Y);
				StopBGM();
			}
			else
				Jiki.IsMuteki = true;
		}

		void HitJudeJikiBurrets()
		{
			foreach(JikiBurret burret in JikiBurrets)
			{
				foreach(EnemyBase enemy in Enemies)
				{
					if(Math.Pow(burret.Radius + enemy.Radius, 2) > Math.Pow(burret.X - enemy.X, 2) + Math.Pow(burret.Y - enemy.Y, 2))
					{
						if(enemy is Boss)
							OnBurretHitBoss((Boss)enemy, burret);
						else
							OnBurretHitEnemy(enemy, burret);

						break;
					}
				}
			}
		}

		void OnBurretHitEnemy(EnemyBase enemy, JikiBurret burret)
		{
			enemy.Life--;
			burret.IsDead = true;

			if(!enemy.IsDead)
				Explosions.Add(new Explosion(burret.X, burret.Y, 0, 0, 0.5f));
			else
			{
				BigExplosion(enemy.X, enemy.Y);
				PlaySeHit();
			}
		}

		void OnBurretHitBoss(Boss boss, JikiBurret burret)
		{
			// ボスの戦闘準備が完了していない場合はノーカウント
			if(!boss.IsReady)
				return;

			boss.Life--;
			burret.IsDead = true;

			if(boss.IsDead)
			{
				// ボスを倒した場合はボスを大爆発させる
				OnBossDead(boss);
				PlaySeHit();
			}
			else
			{
				// 単なる命中の場合は自機弾丸を中心に小爆発させる
				Explosions.Add(new Explosion(burret.X, burret.Y, 0, 0, 0.5f));
			}
		}

		void ExplosionAllEnemies()
		{
			foreach(EnemyBase enemy in Enemies)
			{
				if(!enemy.IsDead)
				{
					enemy.Life = 0;
					BigExplosion(enemy.X, enemy.Y);
				}
			}
		}

		void ClearBossBurrets()
		{
			foreach(EnemyBurret enemyBurret in EnemyBurrets)
			{
				if(enemyBurret is BossEnemyBurret)
				{
					BossEnemyBurret burret = (BossEnemyBurret)enemyBurret;
					if(!burret.IsMove)
						burret.IsDead = true;
				}
			}
		}

		void OnBossDead(Boss boss)
		{
			// ボスを倒した場合はボスを大爆発させる
			BigExplosionWhenBossDead(boss.X, boss.Y);

			// 対ボス戦時のBGMを停止して長めの爆発音を入れる
			StopBGM();
			PlaySeBossDead();

			// ザコ敵もいっしょに爆発
			ExplosionAllEnemies();

			// 飛ぶまえのボスの弾丸も消す
			ClearBossBurrets();

			// 少し時間をおいてからAfterBossDeadを実行する
			Timer timer = new Timer();
			timer.Interval = 10000;
			timer.Tick += Timer_Tick1;
			timer.Start();

			void Timer_Tick1(object sender, EventArgs e)
			{
				Timer t = (Timer)sender;
				t.Stop();
				t.Dispose();

				AfterBossDead();
			}
		}

		void AfterBossDead()
		{
			IsBattleAntiBoss = false;
			BaseY = 0;
			Jiki.Y = 0;

			PlayBGM1();
		}

		void BigExplosion(float x, float y)
		{
			Explosions.Add(new Explosion(x, y, 0.1f, 0.1f, 0.5f));
			Explosions.Add(new Explosion(x, y, -0.1f, -0.1f, 0.5f));
			Explosions.Add(new Explosion(x, y, -0.1f, 0.1f, 0.5f));
			Explosions.Add(new Explosion(x, y, 0.1f, -0.1f, 0.5f));

			for(int i = 0; i < 10; i++)
			{
				int r = Random.Next(10);
				float rx = (r - 5) * 0.02f;
				r = Random.Next(10);
				float ry = (r - 5) * 0.02f;
				Explosions.Add(new Explosion(x, y, rx, ry, 0.5f));
			}
		}

		void BigExplosionWhenBossDead(float x, float y)
		{
			for(int i = 0; i < 10; i++)
			{
				int r = Random.Next(10);
				float rx = (r - 5) * 0.04f;
				r = Random.Next(10);
				float ry = (r - 5) * 0.04f;
				Explosions.Add(new Explosion2(x, y, rx, ry, 0.5f));
			}
			Explosions.Add(new Explosion2(x, y, 0.2f, 0.2f, .1f));
			Explosions.Add(new Explosion2(x, y, -0.2f, -0.2f, 1f));
			Explosions.Add(new Explosion2(x, y, -0.2f, 0.2f, 1f));
			Explosions.Add(new Explosion2(x, y, 0.2f, -0.2f, 1f));
		}

		void MoveJikiBurrets()
		{
			foreach(JikiBurret burret in JikiBurrets)
			{
				burret.Move();
			}
		}

		bool IsOutOfField(Character character)
		{
			if(character.Y > BaseY + 40 || character.Y < BaseY - 2.0)
				return true;
			else
				return false;
		}

		void JikiShot()
		{
			JikiBurrets.Add(new JikiBurret(Jiki.X, Jiki.Y, 0, 1f, 0.1f));
			JikiBurrets.Add(new JikiBurret(Jiki.X, Jiki.Y, 0.1f, 1f, 0.1f));
			JikiBurrets.Add(new JikiBurret(Jiki.X, Jiki.Y, -0.1f, 1f, 0.1f));
		}

		void CreateEnemies()
		{
			if(Enemies.Count < 10 && Random.Next(100) % 8 == 1)
			{
				int r = Random.Next(800);
				float x = 1f * r / 100 - 4f;

				Enemies.Add(new Enemy1(x, BaseY + 40, 0.5f, 1, 50));
				Enemies.Add(new Enemy2(x, BaseY + 40, 0.5f, 3, 50));
			}
		}

		void MoveEnemies()
		{
			EnemyBase boss = Enemies.FirstOrDefault(x => x is Boss);
			if(boss != null)
			{
				Enemies.AddRange(((Boss)boss).Enemies);
				(((Boss)boss).Enemies).Clear();
			}

			foreach(EnemyBase enemy in Enemies)
			{
				enemy.Move();
			}
		}

		void MoveEnemyBurrets()
		{
			foreach(EnemyBurret burret in EnemyBurrets)
			{
				burret.Move();
			}
		}

		void DrawField()
		{
			GL.Color3(Color.Green);

			// 方眼を描画
			for(int i = -50; i < 50; i++)
			{
				GL.Normal3(Vector3.UnitZ);
				GL.Begin(BeginMode.Lines);
				{
					GL.Vertex3(2 * i, BaseY + 200, -0.1);
					GL.Vertex3(2 * i, BaseY - 1, -0.1);
				}
				GL.End();
			}

			int first = (int)BaseY / 4 * 4;
			for(int i = 0; i < 100; i++)
			{
				GL.Normal3(Vector3.UnitZ);
				GL.Begin(BeginMode.Lines);
				{
					GL.Vertex3(50, first + 2 * i, -0.1);
					GL.Vertex3(-50, first + 2 * i, -0.1);
				}
				GL.End();
			}
			GL.Color3(Color.White);
		}

		void InitGame()
		{
			Enemies.Clear();
			EnemyBurrets.Clear();
			JikiBurrets.Clear();
			Explosions.Clear();

			IsBattleAntiBoss = false;
			BaseY = 0;
			Jiki = new Jiki(0.5f, Speed);
			Score = 0;

			PlayBGM1();
		}

		/// <summary>
		/// 実際に描画する
		/// </summary>
		private void glControlEx1_Paint(object sender, PaintEventArgs e)
		{
			GL.Clear(ClearBufferMask.ColorBufferBit | ClearBufferMask.DepthBufferBit);

			if(Jiki != null && Jiki.IsBackColorRed)
				GL.ClearColor(Color.Red);
			else
				GL.ClearColor(Color.Black);

			//Lighting();
			GL.Disable(EnableCap.Lighting);
			Draw3D();
			Draw2D();

			glControl.SwapBuffers();
		}

		void Draw2D()
		{
			GL.Disable(EnableCap.Lighting);
			ShowScore();
			ShowJikiLife();
		}

		void Draw3D()
		{
			GL.Enable(EnableCap.DepthTest);

			SetProjection();
			SetSight();
			DrawField();

			// 自機を描画
			if(Jiki != null && !Jiki.IsDead)
				Jiki.Draw();

			DrawJikiBurrets();

			// 敵機を描画
			// 自機死亡の場合は表示しない。
			if(Jiki != null && !Jiki.IsDead)
			{
				DrawEnemies();
				EnemiesBurretsDraw();
			}
			DrawExplosions();
		}

		private void SetProjection2()
		{
			// ビューポートの設定
			GL.Viewport(0, 0, glControl.Width, glControl.Height);

			// 視体積の設定（2Dにする）
			GL.MatrixMode(MatrixMode.Projection);
			float h = 1.0f, w = h * glControl.AspectRatio;
			Matrix4 proj = Matrix4.CreateOrthographic(w, h, 0.01f, 1f);
			GL.LoadMatrix(ref proj);

			// MatrixMode を元に戻す
			GL.MatrixMode(MatrixMode.Modelview);
		}

		void SetSight2()
		{
			// 2Dのときの視界の設定
			Vector3 eye = new Vector3(0, 0, 0.1f);
			Vector3 target = new Vector3(0, 0, 0);
			Vector3 up = Vector3.UnitY;

			Matrix4 look = Matrix4.LookAt(eye, target, up);
			GL.LoadMatrix(ref look);
		}

		void ShowScore()
		{
			SetProjection2();
			SetSight2();

			float h = 1.0f, w = h * glControl.AspectRatio;
			Bitmap bitmap = GetBitmapForScore();

			int texture = CreateTexture(bitmap);

			GL.PushMatrix();
			{
				GL.Translate(0, h / 2, 0);
				GL.BindTexture(TextureTarget.Texture2D, texture);
				GL.Begin(BeginMode.Quads);
				{
					float d = 0f;
					GL.Normal3(Vector3.UnitZ);
					GL.TexCoord2(0, 0);
					GL.Vertex3(-w / 2, 0, d);

					GL.TexCoord2(1, 0);
					GL.Vertex3(w / 2f, 0, d);

					GL.TexCoord2(1, 1);
					GL.Vertex3(w / 2f, -h * bitmap.Height / bitmap.Width, d);

					GL.TexCoord2(0, 1);
					GL.Vertex3(-w / 2, -h * bitmap.Height / bitmap.Width, d);
				}
				GL.End();
				GL.BindTexture(TextureTarget.Texture2D, 0);
			}
			GL.PopMatrix();

			if(oldTextureForScore != 0)
				GL.DeleteTexture(oldTextureForScore);

			oldTextureForScore = texture;
		}

		void ShowJikiLife()
		{
			SetProjection2();
			SetSight2();

			float h = 1.0f, w = h * glControl.AspectRatio;
			Bitmap bitmap = GetBitmapForJikiLife();
			int texture = CreateTexture(bitmap);

			GL.PushMatrix();
			{
				GL.Translate(0, -h / 2, 0);
				GL.BindTexture(TextureTarget.Texture2D, texture);
				GL.Begin(BeginMode.Quads);
				{
					float d = 0f;

					GL.Normal3(Vector3.UnitZ);
					GL.TexCoord2(0, 0);
					GL.Vertex3(-w / 2, h * bitmap.Height / bitmap.Width, d);

					GL.TexCoord2(1, 0);
					GL.Vertex3(w / 2f, h * bitmap.Height / bitmap.Width, d);

					GL.TexCoord2(1, 1);
					GL.Vertex3(w / 2f, 0, d);

					GL.TexCoord2(0, 1);
					GL.Vertex3(-w / 2, 0, d);
				}
				GL.End();
				GL.BindTexture(TextureTarget.Texture2D, 0);
			}
			GL.PopMatrix();

			if(oldTextureForJikiLife != 0)
				GL.DeleteTexture(oldTextureForJikiLife);

			oldTextureForJikiLife = texture;
		}

		int CreateTexture(Bitmap bitmap)
		{
			GL.Enable(EnableCap.Texture2D);

			GL.AlphaFunc(AlphaFunction.Gequal, 0.5f);
			GL.Enable(EnableCap.AlphaTest);

			int texture;
			GL.GenTextures(1, out texture);

			GL.BindTexture(TextureTarget.Texture2D, texture);

			System.Drawing.Imaging.BitmapData data = bitmap.LockBits(
					new Rectangle(0, 0, bitmap.Width, bitmap.Height),
					System.Drawing.Imaging.ImageLockMode.ReadOnly,
					System.Drawing.Imaging.PixelFormat.Format32bppArgb);

			GL.TexImage2D(
				TextureTarget.Texture2D, 0, PixelInternalFormat.Rgba, data.Width, data.Height, 0,
				PixelFormat.Bgra, PixelType.UnsignedByte, data.Scan0);

			bitmap.UnlockBits(data);

			GL.GenerateMipmap(GenerateMipmapTarget.Texture2D);
			GL.BindTexture(TextureTarget.Texture2D, 0);

			return texture;
		}

		Bitmap GetBitmapForScore()
		{
			int margin = 20;

			Bitmap bitmap = new Bitmap(glControl.Width, 100);
			Graphics g = Graphics.FromImage(bitmap);
			g.DrawString(String.Format("Score {0:00000}", Score), new Font("ＭＳ ゴシック", 15, FontStyle.Bold), new SolidBrush(Color.White), new Point(margin, 25));
			Color color = bitmap.GetPixel(0, 0);

			Boss boss = (Boss)Enemies.FirstOrDefault(x => x is Boss);
			if(boss != null && !boss.IsDead && Jiki != null && !Jiki.IsDead)
			{
				float f = 1f * boss.Life / boss.MaxLife;
				int bossMaxLifeWidth = (int)(glControl.Width - margin * 2);
				int bossLifeWidth = (int)(bossMaxLifeWidth * f);
				g.FillRectangle(new SolidBrush(Color.Blue), new RectangleF(margin, 50, bossMaxLifeWidth, 20));
				g.FillRectangle(new SolidBrush(Color.Orange), new RectangleF(margin, 50, bossLifeWidth, 20));
			}
			g.Dispose();

			return bitmap;
		}

		Bitmap GetBitmapForJikiLife()
		{
			int margin = 20;

			Bitmap bitmap = new Bitmap(glControl.Width, 30);

			if(Jiki != null && !Jiki.IsDead)
			{
				Graphics g = Graphics.FromImage(bitmap);
				int jikiMaxLifeWidth = (int)(glControl.Width * 0.5f);
				int jikiLifeWidth = (int)(jikiMaxLifeWidth * (1f * Jiki.Life / Jiki.MaxLife));
				g.FillRectangle(new SolidBrush(Color.Blue), new RectangleF(margin, 0, jikiMaxLifeWidth, 20));
				g.FillRectangle(new SolidBrush(Color.Orange), new RectangleF(margin, 0, jikiLifeWidth, 20));
				g.Dispose();
			}

			return bitmap;
		}

		void DrawExplosions()
		{
			foreach(Explosion explosion in Explosions)
			{
				explosion.Draw();
			}
		}

		void DrawJikiBurrets()
		{
			foreach(JikiBurret burret in JikiBurrets)
			{
				burret.Draw();
			}
		}

		float _eyeX = 0;
		float _eyeY = 0;

		float EyeX
		{
			set{ _eyeX = value; }
			get{ return _eyeX; }
		}

		float EyeY
		{
			set { _eyeY = value;}
			get { return _eyeY; }
		}

		float _targetY = 0;
		float _targetZ = 0;

		float TargetY
		{
			set { _targetY = value; }
			get { return _targetY; }
		}

		float TargetZ
		{
			set { _targetZ = value;}
			get { return _targetZ; }
		}

		protected override void OnKeyUp(KeyEventArgs e)
		{
			if(Jiki == null)
				return;

			if(e.KeyCode == Keys.Left)
			{
				Jiki.VecX = 0;
			}
			if(e.KeyCode == Keys.Right)
			{
				Jiki.VecX = 0;
			}
			if(e.KeyCode == Keys.Up)
			{
				Jiki.VecY = Speed;
			}
			if(e.KeyCode == Keys.Down)
			{
				Jiki.VecY = Speed;
			}
		}

		protected override void OnKeyDown(KeyEventArgs e)
		{
			if(Jiki == null)
				return;

			if(e.KeyCode == Keys.Left)
			{
				Jiki.VecX = -0.1f;
			}
			if(e.KeyCode == Keys.Right)
			{
				Jiki.VecX = 0.1f;
			}
			if(e.KeyCode == Keys.Up)
			{
				Jiki.VecY = 0.1f + Speed;
			}
			if(e.KeyCode == Keys.Down)
			{
				Jiki.VecY = -0.1f + Speed;
			}
			if(e.KeyCode == Keys.Space)
				JikiShot();

			base.OnKeyDown(e);
		}

		private void MenuItemGameStart_Click(object sender, EventArgs e)
		{
			InitGame();
		}

		private void DrawEnemies()
		{
			Enemies = Enemies.Where(x => !x.IsDead).ToList();
			foreach (EnemyBase enemy in Enemies)
			{
				enemy.Draw();
			}
		}

		void EnemiesBurretsDraw()
		{
			foreach (EnemyBase enemy in Enemies)
			{
				foreach (EnemyBurret burret in EnemyBurrets)
				{
					burret.Draw();
				}
			}
		}

		WMPLib.WindowsMediaPlayer mediaPlayerDamage = new WMPLib.WindowsMediaPlayer();

		void PlaySeDamage()
		{
			string path = Application.StartupPath + "\\damage.mp3";
			if (System.IO.File.Exists(path))
			{
				mediaPlayerDamage.settings.autoStart = true;
				mediaPlayerDamage.URL = path;
			}
		}

		int playSeHitCount = 0;
		WMPLib.WindowsMediaPlayer mediaPlayerHit1 = new WMPLib.WindowsMediaPlayer();
		WMPLib.WindowsMediaPlayer mediaPlayerHit2 = new WMPLib.WindowsMediaPlayer();
		WMPLib.WindowsMediaPlayer mediaPlayerHit3 = new WMPLib.WindowsMediaPlayer();

		void PlaySeHit()
		{
			string path = Application.StartupPath + "\\hit.mp3";
			if (System.IO.File.Exists(path))
			{
				playSeHitCount++;

				WMPLib.WindowsMediaPlayer mediaPlayerHit = mediaPlayerHit1;
				if (playSeHitCount % 2 == 1)
					mediaPlayerHit = mediaPlayerHit2;
				if (playSeHitCount % 2 == 2)
					mediaPlayerHit = mediaPlayerHit3;

				mediaPlayerHit.settings.autoStart = true;
				mediaPlayerHit.URL = path;
			}
		}

		WMPLib.WindowsMediaPlayer mediaPlayerBossDead = new WMPLib.WindowsMediaPlayer();
		void PlaySeBossDead()
		{
			string path = Application.StartupPath + "\\big-explosion.mp3";
			if (System.IO.File.Exists(path))
			{
				mediaPlayerBossDead.settings.autoStart = true;
				mediaPlayerBossDead.URL = path;
			}
		}

		private void StopBGM()
		{
			//再生しているオーディオを停止する
			mediaPlayerBGM.controls.stop();
		}

		DateTime bgm1EndTime = DateTime.Now;
		DateTime bgm2EndTime = DateTime.Now;

		WMPLib.WindowsMediaPlayer mediaPlayerBGM = new WMPLib.WindowsMediaPlayer();
		void PlayBGM1()
		{
			bgm1EndTime = DateTime.Now + new TimeSpan(0, 0, 33);

			string path = Application.StartupPath + "\\battle.mp3";
			if (System.IO.File.Exists(path))
			{
				mediaPlayerBGM.settings.autoStart = true;
				mediaPlayerBGM.URL = path;
			}
		}

		void PlayBGM2()
		{
			bgm2EndTime = DateTime.Now + new TimeSpan(0, 0, 58);

			string path = Application.StartupPath + "\\boss.mp3";
			if (System.IO.File.Exists(path))
			{
				mediaPlayerBGM.settings.autoStart = true;
				mediaPlayerBGM.URL = path;
			}
		}
	}

	public class GLControlEx: OpenTK.GLControl
	{
		public GLControlEx()
		{
			// ないとForm1でKeyDown、KeyUpイベントが捕捉できない
			SetStyle(ControlStyles.Selectable, false);
		}
	}
}
